import * as metadata from "contract-metadata";
export interface AskConfig {
    /**
     * If true, all warnings will be errors when transforming code.
     */
    readonly strict?: boolean;
    /**
     * Chain environment type related config.
     */
    readonly env?: EnvConfig;
    /**
     * Event related config.
     */
    readonly event?: EventConfig;
    /**
     * Contract related infos.
     */
    readonly metadataContract?: metadata.IContract;
}
/**
 * Supported env types this time.
 */
export declare enum EnvType {
    AccountId = "AccountId",
    Balance = "Balance",
    Timestamp = "Timestamp",
    BlockNumber = "BlockNumber"
}
/**
 * Chain environment type related config. Default is same with ink!.
 */
export interface EnvConfig {
    [EnvType.AccountId]?: string;
    [EnvType.Balance]?: string;
    [EnvType.Timestamp]?: string;
    [EnvType.BlockNumber]?: string;
}
/**
 * Event related config. Default to 4.
 */
export interface EventConfig {
    maxTopicNum?: number;
}
/**
 *
 * @returns return a default AskConfig
 */
export declare function defaultConfig(): AskConfig;
export declare function defaultEventConfig(): EventConfig;
export declare function defaultEnvConfig(): EnvConfig;
